/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430FG4618 Experimenter's Board Real Time Clock in Low-power mode
//
//           MSP430FG4618
//         ---------------
//     /|\|            XIN|-
//      | |               | 32kHz
//      --|RST        XOUT|-
//        |               |
//        
//
//  Built with IAR Embedded Workbench Version: 3.42A
//
//  K. Chen
//  Texas Instruments Inc.
//  Ver 2.00 - March 2012
//  Built with IAR Embedded Workbench Version: 5.40.3 & Code Composer Studio V5.1
//******************************************************************************
#include "msp430xG46x.h"

// Function prototypes

void LCD_CHECK(void);
void RTC_Tick(unsigned int TickMins);       // RTC
void RTC_Dec(unsigned int TickMins);        // RTC
void Disp_BCD(unsigned long Value);

// LCD segment definitions.

#define a 0x01
#define b 0x02
#define c 0x04
#define d 0x08
#define e 0x40
#define f 0x10
#define g 0x20
#define h 0x80

const char char_gen[] = {                   // definitions for digits
  a+b+c+d+e+f,                              // Displays "0"
  b+c,                                      // Displays "1"
  a+b+d+e+g,                                // Displays "2"
  a+b+c+d+g,                                // Displays "3"
  b+c+f+g,                                  // Displays "4"
  a+c+d+f+g,                                // Displays "5"
  a+c+d+e+f+g,                              // Displays "6"
  a+b+c,                                    // Displays "7"
  a+b+c+d+e+f+g,                            // Displays "8"
  a+b+c+d+f+g,                              // Displays "9"
  0,                                        // Blank
  0,                                        // Blank
  0,                                        // Blank
  0,                                        // Blank
  0,                                        // Blank
  0                                         // Blank
};



// Global vars for RTC

static unsigned char Secs = 0x00;           // RTC, default time 12:00:00
static unsigned char Mins = 0x00;
static unsigned char Hrs = 0x12;

// Misc

void main(void)
{
  volatile unsigned int i;

  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  FLL_CTL0 |= XCAP14PF;                     // Configure load caps

  // LFXT1 startup delay
  do
  {
    IFG1 &= ~OFIFG;                         // Clear OSCFault flag
    for (i = 0x47FF; i > 0; i--);           // Time for flag to set
  }
  while (IFG1 & OFIFG);                     // OSCFault flag still set?

  
  // Basic Timer
  
  BTCTL = BT_ADLY_1000;                 // 1s BT Interrupt
  IE2 |= BTIE;                          // Enable BT interrupt
  
  // Ports
  
  P1OUT = 0;                            // All P1.x reset
  P1IES = 0x03;                         // P1.0, P1.1 hi/low edge
  P1DIR = 0xFC;                         // P1.0/1 = input (switches)
  P2OUT = 0;                            // All P2.x reset
  P2DIR = 0xDF;                         // UCA0RXD = inp
  P3OUT = 0x20;                         // All P3.x reset (3.5 = buzzer)
  P3DIR = 0xFF;                         // I2C pins = inp
  P4OUT = 0;                            // All P4.x reset
  P4DIR = 0xFF;                         // All P4.x outputs
  P5OUT = 0;                            // All P5.x reset
  P5DIR = 0xFF;                         // All P5.x outputs
  P6OUT = 0;                            // All P6.x reset
  P6DIR = 0xFF;                         // All P6.x outputs
  P7OUT = 0;                            // All P7.x reset
  P7DIR = 0xFF;                         // All P7.x outputs
  P8OUT = 0;                            // All P8.x reset
  P8DIR = 0xFF;                         // All P8.x outputs
  P9OUT = 0;                            // All P9.x reset
  P9DIR = 0xFF;                         // All P9.x outputs
  P10OUT = 0;                           // All P10.x reset
  P10DIR = 0xFF;                        // All P10.x outputs       
  P1IE |=BIT0+BIT1;
  
  // LCD_A
  
  LCDACTL = LCDON + LCD4MUX + LCDFREQ_128;  // 4mux LCD, ACLK/128
  LCDAPCTL0 = 0x7E;                         // Segments 4-27
  P5SEL = 0x1C;
  LCD_CHECK();

  // Misc. initialization
  // The following code is needed to ensure a clean POR-like start
  // when coming out of a WDT reset.
  
  while (1)                                 // repeat forever
  {
    IFG2 &= ~BTIFG;                         // Clear pending interrupt
    IE2 |= BTIE;                            // Enable BT interrupt
    
    // wait for event, hold CPU in low-power mode 3
    __bis_SR_register(LPM3_bits + GIE);
    __disable_interrupt();
    IE2 &= ~BTIE;                           // Disable BT interrupt
  }
}

//------------------------------------------------------------------------------
// LCD Test Code
//------------------------------------------------------------------------------

void LCD_CHECK()
{
  volatile unsigned int i;
  int j;
  unsigned int offset = 2;                  // LCD memory offset variable
  const unsigned int delay = 10000;         // SW delay for LCD stepping

  for( j = 0; j < 12; j ++)
  {
    LCDMEM[j+offset] = 0;                   // Clear LCD
  }

  for( j = 0; j < 11; j ++)
  {
    LCDMEM[j+offset] = 0xFF;                // All segments on
    for (i = delay; i>0; i--);              // Delay
  }

  for (i = delay; i>0; i--);                // Delay

  for( j = 0; j < 12; j ++)
  {
    LCDMEM[j+offset] = 0;                   // Clear LCD
  }
}

//------------------------------------------------------------------------------
// Advance RTC counter variables
//
// TickMins == 0 --> advance seconds
// TickMins == 1 --> advance minutes
//------------------------------------------------------------------------------

void RTC_Tick(unsigned int TickMins)
{
  Secs = __bcd_add_short(Secs, 1);
  if ((Secs == 0x60) || TickMins)
  {
    Secs = 0x00;
    Mins = __bcd_add_short(Mins, 1);
    if (Mins == 0x60)
    {
      Mins = 0x00;
      Hrs = __bcd_add_short(Hrs, 1);
      if (Hrs == 0x13)
        Hrs = 0x01;
    }
  }
}

//------------------------------------------------------------------------------
// Decrement RTC counter variables
//
// TickMins == 0 --> decrement seconds
// TickMins == 1 --> decrement minutes
//------------------------------------------------------------------------------

void RTC_Dec(unsigned int TickMins)
{
  Secs = __bcd_add_short(Secs, 0x9999);
  if ((Secs == 0x99) || TickMins)
  {
    Secs = TickMins ? 0x00 : 0x59;
    Mins = __bcd_add_short(Mins, 0x9999);
    if (Mins == 0x99)
    {
      Mins = 0x59;
      Hrs = __bcd_add_short(Hrs, 0x9999);
      if (Hrs == 0x00)
        Hrs = 0x12;
    }
  }
}

//------------------------------------------------------------------------------
// Displays the BCD number 'Value' on the LCD display. 7 digits are output.
//------------------------------------------------------------------------------

void Disp_BCD(unsigned long Value)
{
  char *pLCD = (char *)&LCDM3;
  int i;

  for (i = 0; i < 7; i++)                       // Process 7 digits
  {
    *pLCD++ = char_gen[Value & 0x0f];           // Segments to LCD
    Value >>= 4;                                // Process next digit
  }
}

//------------------------------------------------------------------------------
// PORT1 interrupt handler
//------------------------------------------------------------------------------

#pragma vector=PORT1_VECTOR
__interrupt void Port1_ISR (void)
{
  volatile unsigned int i, j;

      for (i = 10000; i > 0; i--);          // Button debounce delay
      j = 10000;                            // Initial delay
      do {
        if (!(P1IN & 0x01)) RTC_Tick(1);
        if (!(P1IN & 0x02)) RTC_Dec(1);
        Disp_BCD(0x0f + ((long)Hrs << 20) + ((long)Mins << 12) + (Secs << 4));
        for (i = j; i > 0; i--);            // Variable delay
        if (j > 1200) j -= 200;
      } while ((P1IN & 0x03) != 0x03);
      P1IFG &= ~0x03;                       // P1.0/P1.1 IFGs Cleared
}   

//------------------------------------------------------------------------------
// Basic Timer interrupt service routine
//------------------------------------------------------------------------------

#pragma vector=BASICTIMER_VECTOR
__interrupt void basic_timer(void)
{
 volatile unsigned int i;

      RTC_Tick(0);                          // Advance RTC seconds
      Disp_BCD(0x0f + ((long)Hrs << 20) + ((long)Mins << 12) + (Secs << 4));
      LCDM7 |= 0x80;                        // Enable left colon
      if (Secs & 0x01) LCDM5 |= 0x80;       // Toggle right colon      
}
